
--=============================================================================
-- Eligium game compatable skill trainer
-- For the MacroMonkey automation,macro,bot scripting system.
-- Version: 1.0
-- By Sirmabus Febuary 2012
-- www.macromonkey.com
--
--=============================================================================

--[[
Use this MacroMonkey script to help you build up your skill levels.
Can be used to build attack skills against a Sandbag (training dummy) or to
train buffing and healing skills up, etc.
Basically it's a skill spamming macro script with a UI.

How to use:
1) Load up the Eligium client and log-in to your character to train.
   At least temporary set the client to windowed mode.

2) Go to a relative safe spot and, or, the nearest Sandbag for training attack
   skills.

3) Setup your skill hot keys and delays.
   The delay is to account for skill cooldowns and to allow some control over
   mana regeneration.

   Examples:
   A) You are a mage class and you set up your fireball attack skill in hot key
   slot #1.
   Well set the first script UI setting for the key '1'.
   The cooldown is 1.4 seconds so we set it in the UI as such.

   Now when we follow the next steps, we will stand there and shoot off one
   fireball after another as fast as we are allowed.

   B) With this mage class again we want to build up our "Mana Spring" buff too.
   Lets put in in hot bar slot #8.
   Well set the 2nd script UI setting for the key '8'.
   It's cooldown is 61.3 so we'll enter that in the script UI.
   But to account for unknown delays and possibly lag it would be better to increase
   that a bit to say 61.5 seconds.
   Else it might miss some times, and then we'd have to wait a whole cool down delay
   to try again.

   Now when we follow the next steps, we will stand there and shoot off one
   fireball after another and do the "Mana Spring" buff about every 61.3 seconds.

   You will probably run out of mana after several minutes.
   So what you can do, if you have the gold to spare, is buy stacks of MP (mana potion).
   Then put a link to the MP in hotbar slot and set up a trainer setting slot accordingly.
   The tricky part will be tweaking the cooldown time.
   Too fast you will be burning up MP pots, too slow then you will be out of mana for
   some time. It will, depending on your mana usage and the MP pot(s), be in the typical
   range of 30 seconds to several minutes.
   This setup is currently pretty dumb (lacking game feedback). Ideally it would read your
   current mana level and use a MP pot automatically when it drops below a set level.
   You'll have to experiment and tweak the cooldown.

   The "Enable mouse clicks" option:
   ** Read carfully **
   You only need this feature for click target type skills (mainly certain AOEs).
   Since this script is a macro (not a bot with more facilities) there are certain limitations.
   The game has an auto-move feature that can't be disabled so we have to be particularly
   careful of how we do automated mouse clicks.
   First the game client must be set to 3D view mode and tilted for a pure overhead view.
   This because we will be clicking on the exact center of the client window.
   Hopefully this should keep you from click moving.
   If you are not careful you will do some kind of weird random moves, end up in some odd location,
   or something even worse (and will most likely look like a bot to other players).
   With out this option you can run the skill trainer with the game minimized, you can still with this
   but you will not be able to use your computer because the script must move your mouse to a physical
   location for each skill use.

   Click the "Save Settings" button to save your skill settings and save the current dialog position.

4) If you are going to be doing attack skills get close enough to a Sandbag first, then
   target it and start auto-attack on it (either double click, or press the AA key).
   This is important as you'll need an attackable object to train attack skills on.
   Or, if you are doing buffs or heals you might have to target your self (CTRL + ~) first.

5) Click start and train away.
   Go watch a movie, play another account, etc., while you build up your skill(s), AFK or not.
   But keep an eye on it. You might hit your maxium skill level(s) before you know it.
   Due to the skill or player level limits, etc.

Note:
There is a anti-cheat system in the game. As of this writing MacroMonkey is not it's radar
(read, not being detected).
If at any time you get kicked out of the game with a temporary block notice or similar then stop
using the script immediately.

You take all responsibility and liability for using, and, or, possessing this script et al.
If you get an account ban, loose a limb or appendage, your computer has a meltdown,
you spill scalding coffee on your crotch, or anything in between then you can't blame me for it.

--]]


---------- Utility stuff ----------

-- Cache some functions local for speed
local format = string.format
local modf   = math.modf

-- C function printf
function printf(...)
   local function wrapper(...) io.write(format(...))  end
   local status, result = pcall(wrapper, ...)
   if not status then error(result, 4) end
end

--  C function printf with fore and background colors
function printfc(foreground, background, ...)
   local szText = nil
   local function wrapper(...)
      console.SetColor(foreground, background)
      io.write(format(...))
      console.SetColor(console.WHITE, console.BLACK)
   end
   local status, err = pcall(wrapper, ...)
   if not status then error(err, 2) end
end

-- Return a pretty time string in days/hours/minute/seconds
function TimeString(fSeconds)
   local szTime = nil

   local Days = (fSeconds / time.DAY)
   if (Days >= 1) then
      local iDays = modf(Days)
      if (iDays == 1) then
         szTime = "1 day"
      else
         szTime = format("%d days", iDays)
      end
      fSeconds = (fSeconds - (iDays * time.DAY))
   end

   local Hours = (fSeconds / time.HOUR)
   if (Hours >= 1) then
      if szTime then szTime = (szTime..", ") else szTime = "" end
      local iHours,fHours = modf(Hours)
      if (fHours < 0.1) then
         if (iHours == 1) then
            return(szTime.."1 hour.")
         else
            return(szTime..format("%d hours", iHours))
         end
      end
      return(szTime..format("%.1f hours", Hours))
   end

   local Minutes = (fSeconds / time.MINUTE)
   if (Minutes >= 1) then
      if szTime then szTime = (szTime..", ") else szTime = "" end
      local iMinutes,fMinutes = modf(Minutes)
      if (fMinutes < 0.1) then
         if (iMinutes == 1) then
            return(szTime.."1 minute")
         else
            return(szTime..format("%d minutes", iMinutes))
         end
      else
         return(szTime..format("%.1f minutes", Minutes))
      end
   end

   if szTime then szTime = (szTime..", ") else szTime = "" end
   local iSeconds = modf(fSeconds)
   if (iSeconds == 1) then
      return(szTime.."1 second")
   else
      return(szTime..format("%d seconds", iSeconds))
   end

   szTime = (szTime..".")
   return(szTime)
end

-- Get a current directory path string
function GetCurrentDirectory() return(io.popen("cd"):read()) end

-- Filter editbox characters to fit for a floating point number
local ZeroChar = string.byte("0")
function FilterEditBoxFloatNumber(eb)
   local szText = eb:GetText()

   -- Can't be a null string
   if (not szText or (string.byte(szText) == 0)) then
      -- Fix it
      eb:SetText("0")
      audio.PlaySound()
      return(true)
   else
      -- Filter out any unwanted characters
      local szText, nTimes = szText:gsub("[^0123456789.]", "")
      if (nTimes > 0) then
         eb:SetText(szText)
         audio.PlaySound()
         return(true)
      end

      -- Can have only one '.'
      local pos = szText:find(".", 1, true)
      if pos then
         local pos = szText:find(".", pos+1, true)
         if pos then
            szText = szText:sub(1, pos-1)
            eb:SetText(szText)
            audio.PlaySound()
            return(true)
         end
      end

      -- Filter out leading zero
      if ((#szText > 1) and (szText:byte() == ZeroChar)) then
         szText = szText:sub(2, pos-1)
         eb:SetText(szText)
         audio.PlaySound()
         return(true)
      end
   end

   return(false)
end

---------- Main ----------
console.Clear()
local LastElapsedUpdate = -math.huge
local GlobalCooldown    = -math.huge
local LastBarColor,LastBarPos = -1,-1
-- Global cooldown time
-- It appears to be normally exaclty one second, but needs to be higher when
-- doing multiple skills.
local cGlobalCooldownTime = 1.6

-- Custom console color palette
local RGB = color.RGB
local aPalette =
{
   RGB(0,0,0),       -- Black
   RGB(0,0,170),     -- Blue
   RGB(0,170,0),     -- Green
   RGB(0,170,170),   -- Cyan
   RGB(170,0,0),     -- Red
   RGB(170,0,170),   -- Magenta
   RGB(170,85,0),    -- Yellow
   RGB(243,233,199), -- White RGB(170,170,170)
   RGB(85,85,85),    -- Grey
   RGB(85,85,255),   -- Bright blue
   RGB(139,254,2),   -- Bright green RGB(85,255,85)
   RGB(133,85,255),  -- Bright cyan RGB(85,255,255)
   RGB(255,37,1),    -- Bright red RGB(255,85,85)
   RGB(255,85,255),  -- Bright magenta
   RGB(255,240,1),   -- Bright yellow RGB(255,255,85)
   RGB(255,255,255)  -- Bright white
}
console.SetColorTable(aPalette)


-- Find the game window
-- TODO: Could handle multiple client windows here
local hWndGame = win.Find("Eligium", "Gamebryo Application", nil,nil, win.FIND_TILE_INSIDE)
if hWndGame then
   --printf("Game win: %Xh\n", hWndGame)
   local DialogColor = color.RGB(231, 102, 7)
   local CForeColor  = color.RGB(0, 0, 0)
   local CBackColor  = color.RGB(244, 234, 198)

   -- Get saved dialog position
   local szINIPath = (GetCurrentDirectory() .. "\\SkillSettings.ini")
   local x = ini.GetValue("Dialog", "x", szINIPath)
   local y = ini.GetValue("Dialog", "y", szINIPath)
   if not x or not y then x,y = 0x80000000,0x80000000 end

   -- Create dialog window
   local Dialog = gui.Window("Eligium Skill Trainer", x,y, 196,455, bit.bor(gui.Window.STYLE_DEF, gui.Window.NOUSERCLOSE, gui.Window.WS_THICKFRAME, gui.Window.WS_VISIBLE),nil, 0, DialogColor)
   assert(Dialog)
   Dialog:SetState(gui.Window.DISABLE)
   -- Make the default activation delays a little longer
   input.KeySetDelay(0.050)
   input.MouseSetDelay(0.050)

   -- Place console below window
   local function UpdateConsolePos()
      local x,y = Dialog:GetPos()
      local w,h = Dialog:GetSize()
      win.SetPos(console.GetWindow(), x, (y + h))
   end
   local hWndCon = console.GetWindow()
   local h,w = Dialog:GetClientSize()
   win.SetSize(hWndCon, (w - 235), 228)
   win.SetText(hWndCon, "Log")
   UpdateConsolePos()

   -- Load, or make default skill bar settings
   local aSkillBar = {} -- [1] = Key, [2] = Cooldown time, [3] = Last use time, [4] = GUI container
   for i = 1,6 do
      local t = {}
      local s = ini.GetValue("SkillBars", tostring(i), szINIPath)
      if s then
         local v1,v2 = s:match("(%d+) ([%d%.]+)")
         t[1],t[2] = tonumber(v1),tonumber(v2)
      else
         t[1],t[2] = 0, 2.5
      end
      aSkillBar[i] = t
   end

   -- Animated global skill cooldown tracking bar
   local x,y = 8,9
   CDTrackingBar = gui.ProgressBar(Dialog, 11,y, 159,11, gui.Window.WS_VISIBLE)
   assert(CDTrackingBar)
   CDTrackingBar:SetColor(color.RGB(255, 235, 63), DialogColor)
   CDTrackingBar:SetBarPos(0)

   -- Elasped time box
   y = (y + 23)
   local StartTime = nil
   local TimeLabel = gui.StaticText(Dialog, "Running:", x,(y + 1), 24,24, nil,nil, CBackColor)
   assert(TimeLabel)
   local TimeBox   = gui.EditBox(Dialog, "", (x + 64),y, 87,20, bit.bor(gui.Window.WS_VISIBLE, gui.EditBox.ES_LEFT, gui.EditBox.ES_AUTOHSCROLL, gui.EditBox.ES_READONLY),nil, CForeColor,CBackColor)
   assert(TimeBox)
   TimeBox:SetState(gui.Window.DISABLE)

   -- Add skill bar controls
   y = (y + 33)
   local BarsLabel = gui.StaticText(Dialog, "---- Key -- Cooldown -", 16,y, 24,24, nil,nil, CBackColor)
   assert(BarsLabel)
   local SBX,SBY = 14,(y + 21)
   local SBW,SBH = 45,32
   local aKeyIndex2Str  = {" ","1","2","3","4","5","6","7","8","9","0","F1","F2","F3","F4","F5","F6","F7","F8","F9"}
   local aIndex2KeyCode = {"1","2","3","4","5","6","7","8","9","0",input.VK_F1,input.VK_F2,input.VK_F3,input.VK_F4,input.VK_F5,input.VK_F6,input.VK_F7,input.VK_F8,input.VK_F9}
   local function AddSkillBar(Index)
      local t = {}
      local x,y = SBX,(SBY+((Index - 1) * SBH))
      t.fl = gui.StaticText(Dialog, format("%u -", Index), x,y+4, 24,24, nil,nil, CBackColor)
      assert(t.fl)
      x = (x+21)
      t.cb = gui.ComboBox(Dialog, x,y, SBW,400, nil,nil, CForeColor,CBackColor)
      assert(t.cb)
      x = (x+SBW+6)
      for _,s in ipairs(aKeyIndex2Str) do t.cb:Add(s) end
      t.cb:SetSelection(aSkillBar[Index][1])
      t.eb = gui.EditBox(Dialog, format("%.1f", aSkillBar[Index][2]), x,y, 54,24, nil,nil, CForeColor,CBackColor)
      assert(t.eb)
      x = (x+54+4)
      t.bl = gui.StaticText(Dialog, "-", x,y+4, 24,24, nil,nil, CBackColor)
      assert(t.bl)
      return(t)
   end
   for i = 1,6 do
      aSkillBar[i][3] = -math.huge
      aSkillBar[i][4] = AddSkillBar(i)
   end

   printfc(console.BRIGHTYELLOW, console.BLACK, "Press start button to\n")
   printfc(console.BRIGHTYELLOW, console.BLACK, "start training.\n")
   print("See script file for\nmore information.\n")

   -- Mouse click option checkbox
   local x,y = (SBX - 2),((SBY+(6 * SBH))+2)
   local function ShowMouseClickWarnMsg()
      printfc(console.BRIGHTYELLOW, console.BLACK, "* Warning, for mouse\nclicking to work right\nthe game client must\nbe set to 3D view mode\nand the game camera\nset to direct overhead\nview *\n")
   end
   local MouseClickCB = gui.CheckBox(Dialog, "Enable mouse clicks*", x,y, 160,24, nil,nil, CBackColor)
   assert(MouseClickCB)
   local MouseClickEnable = ini.GetValue("Dialog", "MouseClick", szINIPath)
   if not MouseClickEnable then MouseClickEnable = 0 end
   MouseClickEnable = tonumber(MouseClickEnable)
   MouseClickCB:SetCheckState(MouseClickEnable)
   if (MouseClickEnable > 0) then ShowMouseClickWarnMsg() print() end

   -- Save settings button
   y = (y + 31)
   local SaveBtn = gui.PushButton(Dialog, "Save Settings", (x + 18),y, 108,22)
   assert(SaveBtn)

   -- Show some site hyperlinks
   y = (y + 34)
   local HLForeColor = color.RGB(120, 17, 0)
   local Hyperlink1 = gui.Hyperlink(Dialog, "MM Forum", "http://www.macromonkey.com/bb/viewtopic.php?style=7&f=66&t=855&p=7384#p7384", (x + 18),y, 150,20, nil, nil, nil, HLForeColor)
   assert(Hyperlink1)
   y = (y + 19)
   local Hyperlink2 = gui.Hyperlink(Dialog, "Game site", "http://www.eligium.com/en/", (x + 57),y, 150,20, nil, nil, nil, HLForeColor)
   assert(Hyperlink2)

   -- Start/stop button
   local StartStopBtn = gui.PushButton(Dialog, "Start", (x + 28),(y + 32), 90,23)
   assert(StartStopBtn)
   local StartStopToggle = false

   local KeyPress    = input.KeyPress
   local MouseClick  = input.MouseClick
   local MouseSetPos = input.MouseSetPos
   local Sleep       = time.Sleep
   local PollEvents  = gui.PollEvents
   local GetTime     = time.GetHigh

   -- Game client must be switched into focus to get read for optional mouse clicking
   win.SetState(hWndGame, win.SW_SHOW)
   Sleep(0.200)
   gui.PollEvents()
   local GWinInfo = win.Info(hWndGame)
   win.SetState(hWndCon, win.SW_SHOW)
   Dialog:SetState(gui.Window.ENABLE)
   Dialog:SetState(gui.Window.SHOW)
   Dialog:SetState(gui.Window.FOCUS)

   -- Main loop
   while true do

      -- Poll and update UI
      local Events = PollEvents()
      if Events then
         for _,E in pairs(Events) do

            -- Key controls
            local IsBtnCtrl = false
            for i = 1,6 do
               if (aSkillBar[i][4].cb == E.Object) then
                  if (E.Event == gui.EVENT_SELECT) then
                     aSkillBar[i][1] = E.Selection
                  end
                  IsBtnCtrl = true
                  break
               elseif (aSkillBar[i][4].eb == E.Object) then
                  if (E.Event == gui.EVENT_CHANGE) then
                     FilterEditBoxFloatNumber(E.Object)
                     aSkillBar[i][2] = tonumber(E.Object:GetText())
                     if not aSkillBar[i][2] then aSkillBar[i][2] = 0 end
                     --printf("%.1f\n", aSkillBar[i][2])
                     --E.Object:SetText(format("%.1f", aSkillBar[i][2]))
                  end
                  IsBtnCtrl = true
                  break
               end
            end

            if (IsBtnCtrl == false) then
               -- Dialog window
               if (E.Object == Dialog) then
                  -- Track console to main window
                  if (E.Event == gui.EVENT_MOVE) then
                     UpdateConsolePos()
                  elseif (E.Event == gui.EVENT_SIZE) then
                     if (E.Code == gui.Window.SIZE_RESTORED) then
                        win.SetState(console.GetWindow(), win.SW_RESTORE)
                        UpdateConsolePos()
                     end
                  -- Close console on window close
                  elseif (E.Event == gui.EVENT_CLOSED) then
                     win.Close(console.GetWindow())
                  end

               -- Save settings button
               elseif (E.Object == SaveBtn) then
                  -- Save skill bar settings
                  for i = 1,6 do
                     ini.SetValue("SkillBars", tostring(i), format("%d %.1f", aSkillBar[i][1], aSkillBar[i][2]), szINIPath)
                  end
                  -- Save current dialog position
                  local x,y = Dialog:GetPos()
                  ini.SetValue("Dialog", "x", x, szINIPath)
                  ini.SetValue("Dialog", "y", y, szINIPath)
                  printfc(console.BRIGHTYELLOW, console.BLACK, "Settings saved.\n")
                  -- Save mouse click enable switch
                  ini.SetValue("Dialog", "MouseClick", MouseClickCB:GetCheckState(), szINIPath)

               -- Mouse click enable checkbox
               elseif (E.Object == MouseClickCB) then
                  if (E.Event == gui.EVENT_CLICK) then
                     MouseClickEnable = E.CheckState
                     if (MouseClickEnable > 0) then
                        printfc(console.BRIGHTYELLOW, console.BLACK, "Targeting mouse clicks\nenabled.\n")
                        ShowMouseClickWarnMsg()
                     else
                        printfc(console.BRIGHTYELLOW, console.BLACK, "Targeting mouse clicks\ndisabled.\n")
                     end
                  end

               -- Start/stop button
               elseif (E.Object == StartStopBtn) then
                  -- Start
                  if (StartStopToggle == false) then
                     -- Not if there are no skills enabled
                     local bAnyEnabled = false
                     for i = 1,6 do
                        if (aSkillBar[i][1] > 1) then
                           bAnyEnabled = true
                           break
                        end
                     end

                     if (bAnyEnabled == true) then
                        StartStopToggle = true
                        StartStopBtn:SetText("Stop")
                        console.Clear()
                        printfc(console.BRIGHTGREEN, console.BLACK, "Started.\n")
                        printfc(console.BRIGHTYELLOW, console.BLACK, "Press the stop button\n")
                        printfc(console.BRIGHTYELLOW, console.BLACK, "to stop training.\n\n")
                        TimeBox:SetState(gui.Window.ENABLE)
                        LastBarColor,LastBarPos = -1,-1
                        StartTime = GetTime()
                     else
                        print()
                        printfc(console.BRIGHTRED, console.BRIGHTYELLOW, "No skills are enabled.\nSet up some skills    \nabove first.          \n")
                     end
                  -- Stop
                  else
                     StartStopToggle = false
                     StartStopBtn:SetText("Start")
                     print()
                     printfc(console.BRIGHTRED, console.BLACK, "Stopped.\n")
                     printfc(console.BRIGHTYELLOW, console.BLACK, "Press start button to\n")
                     printfc(console.BRIGHTYELLOW, console.BLACK, "continue training.\n")
                     TimeBox:SetState(gui.Window.DISABLE)
                     CDTrackingBar:SetBarPos(0)
                  end
               else
                  --print(E.Event)
               end
            end
         end
      end

      -- Process skill list
      if (StartStopToggle == true) then
         -- Wait past the global skill cooldown first
         local Time = GetTime()
         local DeltaGCD = (Time - GlobalCooldown)
         if (DeltaGCD >= cGlobalCooldownTime) then

            -- Any slots waiting to be activated?
            for i = 1,6 do
               if (aSkillBar[i][1] > 1) then
                  if (Time >= aSkillBar[i][3]) then
                     -- Activate skill by hotbar key
                     printfc(console.BRIGHTCYAN, console.BLACK, "Pressing '%s'\n", aKeyIndex2Str[aSkillBar[i][1]])
                     KeyPress(aIndex2KeyCode[aSkillBar[i][1] - 1], hWndGame)

                     -- Click skill destination position
                     -- ** Read help above before using this option **
                     if (MouseClickEnable > 0) then
                        -- Give skill some time to activate
                        printfc(console.BRIGHTCYAN, console.BLACK, " Mouse set & click.\n")
                        Sleep(0.200)
                        -- Left click center of game client screen to place area skill
                        local x = (GWinInfo.x + (GWinInfo.width  * 0.5))
                        local y = (GWinInfo.y + (GWinInfo.height * 0.51))
                        MouseSetPos(x, y)
                        MouseClick(0, hWndGame)
                     end

                     -- Update timers
                     local Time = GetTime()
                     -- Games lies, need extra CD delay
                     -- TODO: Tweak this to generically cover all skills
                     aSkillBar[i][3] = (Time + aSkillBar[i][2] + 0.5)
                     GlobalCooldown  = Time
                     break
                  end
               end
            end
         end

         -- Animate cooldown tracking bar
         local Scale = (DeltaGCD / cGlobalCooldownTime)
         if (Scale > 1.0) then Scale = 1.0 end
         local Pos = modf(Scale * 100)
         if (Pos ~= LastBarPos) then
            LastBarPos = Pos
            CDTrackingBar:SetBarPos(Pos)
         end
         local BarHS,BarHE = 24.0, 60.0
         local H = (BarHS + ((BarHE - BarHS) * Scale))
         local BarColor = color.RGB(color.HSLtoRGB(H, 1.0, 0.50))
         if (BarColor ~= LastBarColor) then
            LastBarColor = BarColor
            CDTrackingBar:SetColor(BarColor)
         end

         -- Update elapsed time display once a second
         if(Time >= LastElapsedUpdate) then
            LastElapsedUpdate = (Time + 1.0)
            TimeBox:SetText(" " .. TimeString(time.DeltaHigh(StartTime)))
         end
      end

      -- Give up our thread for a while
      Sleep(0.050)
   end

-- Game client window not found
else
   printfc(console.BRIGHTYELLOW, console.RED, "\n*** Client game window not found! ***\n")
   printf("<Press any key to exit>")
   console.KeyPause()
end
